#! /usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2020, NVIDIA CORPORATION. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

from nvidia_pyindex.utils import clean_nvidia_pyindex_from_config
from nvidia_pyindex.utils import get_configuration_files_by_priority


def uninstall():
    print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
    print("Uninstalling NVIDIA Pip Configuration ...")
    print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")

    for config_file in get_configuration_files_by_priority():
        clean_nvidia_pyindex_from_config(config_file)


def total_clean():
    config_files = get_configuration_files_by_priority()

    print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
    print("Uninstalling NVIDIA Pip Configuration ...")
    print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")

    for config_file in config_files:
        for file in [config_file, "{}.old".format(config_file)]:
            print("Removing configuration file: %s ..." % file)
            try:
                print("Deleting file: `{}` ...".format(file))
                os.remove(file)
                print("Success !\n")
            except:
                print("Pass ...\n")
                pass


def main():
    if len(sys.argv) >= 2:
        if sys.argv[1] == "uninstall":
            uninstall()
        elif sys.argv[1] == "total_clean":
            total_clean()
        else:
            raise RuntimeError("Unknown commandline argument: %s" % sys.argv[1])
    else:
        raise RuntimeError(
            "Incorrect usage: `nvidia_pyindex uninstall`\n."
            "Received: %s" % sys.argv
        )


if __name__ == "__main__":
    sys.exit(main())



#! /usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2020, NVIDIA CORPORATION. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

MAJOR = 1
MINOR = 0
PATCH = 9
PRE_RELEASE = ''
# Use the following formatting: (major, minor, patch, prerelease)
VERSION = (MAJOR, MINOR, PATCH, PRE_RELEASE)

__shortversion__ = '.'.join(map(str, VERSION[:3]))
__version__ = '.'.join(map(str, VERSION[:3])) + '.'.join(VERSION[3:])

__package_name__ = 'nvidia-pyindex'
__contact_names__ = 'Jonathan Dekhtiar'
__contact_emails__ = 'jdekhtiar@nvidia.com'
__homepage__ = 'http://www.nvidia.com/'
__repository_url__ = 'http://www.nvidia.com/'
__download_url__ = 'http://www.nvidia.com/'
__description__ = 'A tool that adds the NVIDIA PIP Index to the environment.'
__license__ = 'Apache2'
__keywords__ = 'nvidia, deep learning, machine learning, supervised learning,'
__keywords__ += 'unsupervised learning, reinforcement learning, logging'



#! /usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2020, NVIDIA CORPORATION. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import copy
from urllib.parse import urlparse
from nvidia_pyindex.utils import clean_nvidia_pyindex_from_config

__all__ = [
    "create_pip_config_file",
    "maybe_edit_pip_config_file"
]

NVIDIA_PyPI_SERVERS = [
    "https://pypi.ngc.nvidia.com",
]


def _get_host_from_url(url):
    parsed_uri = urlparse(url)
    return '{uri.netloc}'.format(uri=parsed_uri)


def _format_extra_index_url(url):
    return "%s%s\n" % (" " * 18, url)


def _format_trusted_host(url):
    return "%s%s\n" % (" " * 15, _get_host_from_url(url))


def create_pip_config_file(filepath, config_filedict):

    try:
        print("Creating dir:", os.path.dirname(filepath))
        os.makedirs(os.path.dirname(filepath))
    except FileExistsError:
        pass

    content = generate_configfile_header(config_filedict)
    with open(filepath, 'w') as _file:
        content += """[global]
no-cache-dir = true
index-url = https://pypi.org/simple
extra-index-url =
{extra_index_urls}
trusted-host =
{trusted_hosts}
"""
        extra_index_urls = list()
        trusted_hosts = list()

        for nvidia_pypi_server in NVIDIA_PyPI_SERVERS:
            extra_index_urls.append(_format_extra_index_url(nvidia_pypi_server))
            trusted_hosts.append(_format_trusted_host(nvidia_pypi_server))

        extra_index_urls[-1] = extra_index_urls[-1].rstrip('\n')
        trusted_hosts[-1] = trusted_hosts[-1].rstrip('\n')

        _file.write(
            content.format(
                extra_index_urls="".join(extra_index_urls),
                trusted_hosts="".join(trusted_hosts),
            )
        )


def generate_configfile_header(config_filedict):
    header = """# This file has been autogenerated or modified by NVIDIA PyIndex.
# In case you need to modify your PIP configuration, please be aware that
# some configuration files may have a priority order. Here are the following 
# files that may exists in your machine by order of priority:\n"""

    # Priority order
    # https://github.com/pypa/pip/blob/master/src/pip/_internal/configuration.py#L47
    # OVERRIDE_ORDER = kinds.GLOBAL, kinds.USER, kinds.SITE, kinds.ENV, kinds.ENV_VAR

    header += "#\n# [Priority 1] Site level configuration files\n"
    for idx, file in enumerate(config_filedict["site"][::-1]):
        header += "#\t{idx}. `{file}`\n".format(idx=idx+1, file=file)

    header += "#\n# [Priority 2] User level configuration files\n"
    for idx, file in enumerate(config_filedict["user"][::-1]):
        header += "#\t{idx}. `{file}`\n".format(idx=idx+1, file=file)

    header += "#\n# [Priority 3] Global level configuration files\n"
    for idx, file in enumerate(config_filedict["global"][::-1]):
        header += "#\t{idx}. `{file}`\n".format(idx=idx+1, file=file)

    header += "\n"

    return header


def maybe_edit_pip_config_file(filepath, config_filedict, debug_mode=False):
    
    clean_nvidia_pyindex_from_config(filepath)
    
    with open(filepath, 'r') as _file:
        original_content = _file.readlines()

    # Force `no-cache-dir = true`
    for i in range(len(original_content)):
        if "no-cache-dir" in original_content[i]:
            original_content[i] = "no-cache-dir = true\n"
            try:
                if original_content[i+1][0] == " ":
                    del original_content[i+1]
            except IndexError:
                pass
            break
    else:
        original_content.append("no-cache-dir = true\n")

    # Add the `extra-index-url(s)` and `trusted-host` for nvidia python packages
    for nvidia_pypi_server in NVIDIA_PyPI_SERVERS:

        def edit_pip_environment(env_name, value_to_add):
            temp = []
            file_content = copy.deepcopy(original_content)
            try:
                while file_content:
                    line = file_content.pop(0)
                    if env_name in line:
                        temp.append(line)
                        temp.append(value_to_add)
                        while True:
                            line = file_content.pop(0)
                            temp.append(line)
                            if line[0] != " ":
                                break
                        temp += file_content
                        break
                    else:
                        temp.append(line)
                # the env_name wasn't found in the existing config file
                else:
                    temp += [
                        "{} =\n".format(env_name),
                        value_to_add
                    ]
            except IndexError:
                pass

            return temp

        original_content = edit_pip_environment(
            env_name="extra-index-url",
            value_to_add=_format_extra_index_url(nvidia_pypi_server)
        )

        original_content = edit_pip_environment(
            env_name="trusted-host",
            value_to_add=_format_trusted_host(nvidia_pypi_server)
        )

        if not any("autogenerated or modified by NVIDIA PyIndex" in line for line in original_content):
            original_content = (
                generate_configfile_header(config_filedict) +
                "".join(original_content)
            )

    with open(filepath, 'w') as _file:
        _file.writelines(original_content)



import subprocess
import shlex
import json
import sys


def get_configuration_files():
    command = '{py_binary_path} -c "from pip._internal.configuration import ' \
              'get_configuration_files; ' \
              'print(get_configuration_files())"'.format(
                  py_binary_path=sys.executable
              )

    command = shlex.split(command)

    if sys.platform == "win32":
        command[0] = sys.executable  # otherwise there's a bug that removes `\`

    try:
        output = subprocess.check_output(command, stderr=subprocess.STDOUT)
        output = output.decode()
    except subprocess.CalledProcessError as e:
        output = e.output.decode()
        raise RuntimeError(output) from e

    output = output.replace("'", '"')
    output = json.loads(output)

    return output


def get_configuration_files_by_priority():
    config_files = get_configuration_files()

    config_priority_list = []
    config_priority_list += config_files["site"][::-1]
    config_priority_list += config_files["user"][::-1]
    config_priority_list += config_files["global"][::-1]

    return config_priority_list


def is_nvidia_pyindex(line):
    return any([
        pattern in line
        for pattern in [
            "sqrl.nvidia.com",
            "pypi.ngc.nvidia.com",
        ]
    ])


def clean_nvidia_pyindex_from_config(filepath):
    try:
        print("==============================================================")
        print("Cleaning previous NVIDIA PyIndex Configuration ...")
        print("File: `{}`".format(filepath))

        with open(filepath, 'r') as _file:
            file_content = _file.readlines()

        def edit_config_file(env_name, _file_content):
            # Removing all NVIDIA changes
            temp = []
            try:
                while _file_content:
                    line = _file_content.pop(0)
                    if env_name in line:
                        if is_nvidia_pyindex(line):
                            line = "{} = \n".format(env_name)
                        temp.append(line)
                        while True:
                            line = _file_content.pop(0)
                            if line[0] != " ":
                                temp.append(line)
                                break
                            elif not is_nvidia_pyindex(line):
                                temp.append(line)
                    else:
                        temp.append(line)
            except IndexError:
                pass

            return temp

        file_content = edit_config_file("extra-index-url", file_content)
        file_content = edit_config_file("trusted-host", file_content)
        file_content = edit_config_file("no-cache-dir", file_content)

        with open(filepath, 'w') as _file:
            _file.writelines(file_content)

        print("Previous NVIDIA PyIndex Configuration cleaned with success ...")
        print("==============================================================")

    except (FileNotFoundError, PermissionError) as e:
        print("Error while removing the old NVIDIA PyINDEX configuration:", e)

    print("==============================================================")



#! /usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2020, NVIDIA CORPORATION. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

import codecs
import shlex
import subprocess
import time

from pathlib import Path

from setuptools import setup
from setuptools import find_packages
from setuptools.command.develop import develop
from setuptools.command.install import install
from setuptools.command.install_scripts import install_scripts

from package_info import __version__

from package_info import __package_name__
from package_info import __contact_names__
from package_info import __contact_emails__
from package_info import __repository_url__
from package_info import __download_url__
from package_info import __description__
from package_info import __license__
from package_info import __keywords__

from edit_pip_configuration import create_pip_config_file
from edit_pip_configuration import maybe_edit_pip_config_file

from nvidia_pyindex.utils import get_configuration_files
from nvidia_pyindex.utils import get_configuration_files_by_priority

SUPPORTED_PLATFORMS = [
    "linux",
    "win32",  # Windows
    "darwin",  # Mac OS X
]


def _install_nvidia_pypi_index():
    try:
        subprocess.call(
            shlex.split("nvidia_pyindex uninstall"),
            shell=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL
        )
    except:
        pass

    debug_mode = bool(os.environ.get("DEBUG_MODE", False))

    config_filedict = get_configuration_files()
    config_filelist = get_configuration_files_by_priority()

    print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
    print("Setting User Pip Configuration ...")
    print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
    print("\n######################\n")

    pre_existing_configfile = None
    for config_file in config_filelist:
        try:
            if os.path.isfile(config_file):
                print("Editing pip conf file: %s ..." % config_file)
                maybe_edit_pip_config_file(
                    config_file,
                    config_filedict=config_filedict,
                    debug_mode=debug_mode
                )
                pre_existing_configfile = config_file
                break
        except (FileNotFoundError, PermissionError) as e:
            print("Error: {}: {}".format(e.__class__.__name__, str(e)))
            pass

    if pre_existing_configfile is None:
        for config_file in config_filelist:
            print("\nProcessing pip conf file: %s ..." % config_file)
            try:
                if not os.path.exists(config_file):
                    print("Creating pip conf file: %s ..." % config_file)
                    create_pip_config_file(
                        filepath=config_file,
                        config_filedict=config_filedict
                    )
            except (FileNotFoundError, PermissionError) as e:
                print("Error: {}: {}".format(e.__class__.__name__, str(e)))
                pass
    else:
        with open(pre_existing_configfile, 'r') as _file:
            original_content = _file.readlines()

        for config_file in config_filelist:
            print("\nProcessing pip conf file: %s ..." % config_file)
            try:

                try:
                    with open(config_file, 'r') as _file:
                        backup_content = _file.readlines()
                except:
                    pass

                try:
                    os.remove(config_file)
                    os.remove(config_file.old)
                    with open("%s.old" % config_file, 'w') as _file:
                        _file.writelines(backup_content)
                except:
                    pass

                with open(config_file, 'w') as _file:
                    _file.writelines(original_content)

            except (FileNotFoundError, PermissionError) as e:
                print("Error: {}: {}".format(e.__class__.__name__, str(e)))
                pass

    print("\n######################\n")

    time.sleep(2)  # allow readable logs in verbose logs. Please do not delete.


def _force_clean_cache_whl(pkg_name):
    print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
    print("Cleaning Potential Wheel Cache ...")
    print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")

    if sys.argv[1] == "sdist":
        return

    pkg_names = [pkg_name, pkg_name.replace("-", "_")]

    import appdirs
    for path in Path(appdirs.user_cache_dir("pip/wheels")).rglob('*.whl'):
        if any([pkg in path.name for pkg in pkg_names]):
            print("Removing: `%s`" % path)
            path.unlink()

    time.sleep(2)  # allow readable logs in verbose logs. Please do not delete.


class InstallCommand(install):
    """A class for "pip install".
    Handled by "pip install rpm-py-installer",
    when the package is published to PyPI as a source distribution (sdist).
    """

    def run(self):
        """Run install process."""
        print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
        print("COMMAND: %s" % InstallCommand.__name__)
        print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
        _install_nvidia_pypi_index()
        super(InstallCommand, self).run()
        _force_clean_cache_whl(__package_name__)


class DevelopCommand(develop):
    """A class for setuptools development mode.
    Handled by "pip install -e".
    """

    def run(self):
        """Run install process with development mode."""
        print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
        print("COMMAND: %s" % DevelopCommand.__name__)
        print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
        _install_nvidia_pypi_index()
        super(DevelopCommand, self).run()
        _force_clean_cache_whl(__package_name__)


class InstallScriptsCommand(install_scripts):
    """A class for setuptools development mode.
    Handled by "pip install -e".
    """

    def run(self):
        """Run install process with development mode."""
        print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
        print("COMMAND: %s" % InstallScriptsCommand.__name__)
        print("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%")
        super(InstallScriptsCommand, self).run()
        _force_clean_cache_whl(__package_name__)


if __name__ == "__main__":

    # ┍━━━━━━━━━━━━━━━━━━━━━┯━━━━━━━━━━━━━━━━━━━━━━┑
    # │ System              │ `sys.platform` Value │
    # ┝━━━━━━━━━━━━━━━━━━━━━┿━━━━━━━━━━━━━━━━━━━━━━┥
    # │ Linux               │ linux                │
    # │ Windows             │ win32                │
    # │ Windows/Cygwin      │ cygwin               │
    # │ Windows/MSYS2       │ msys                 │
    # │ Mac OS X            │ darwin               │
    # │ OS/2                │ os2                  │
    # │ OS/2 EMX            │ os2emx               │
    # │ RiscOS              │ riscos               │
    # │ AtheOS              │ atheos               │
    # │ FreeBSD 7           │ freebsd7             │
    # │ FreeBSD 8           │ freebsd8             │
    # │ FreeBSD N           │ freebsdN             │
    # │ OpenBSD 6           │ openbsd6             │
    # ┕━━━━━━━━━━━━━━━━━━━━━┷━━━━━━━━━━━━━━━━━━━━━━┙
    if sys.platform not in SUPPORTED_PLATFORMS:
        raise OSError("Your operating system is not supported by "
                      "`nvidia-pyindex`: {}.\nOnly these systems are "
                      "supported: `{}`.".format(
                        sys.platform, SUPPORTED_PLATFORMS
                      )
        )

    # Get the long description
    BASE_DIR = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(BASE_DIR, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()

    setup(
        name=__package_name__,
        version=__version__,
        description=__description__,
        long_description=long_description,
        url=__repository_url__,
        download_url=__download_url__,
        author=__contact_names__,
        author_email=__contact_emails__,
        maintainer=__contact_names__,
        maintainer_email=__contact_emails__,
        license=__license__,
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Developers',
            'Intended Audience :: Science/Research',
            'Intended Audience :: Information Technology',
            'Topic :: Scientific/Engineering',
            'Topic :: Scientific/Engineering :: Image Recognition',
            'Topic :: Scientific/Engineering :: Artificial Intelligence',
            'Topic :: Software Development :: Libraries',
            'Topic :: Utilities',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Environment :: Console',
            'Natural Language :: English',
            'Operating System :: OS Independent',
        ],
        keywords=__keywords__,
        setup_requires=[
            'appdirs>=1.4,<1.5',
        ],
        cmdclass={
            'install': InstallCommand,
            'develop': DevelopCommand
        },
        entry_points={
            'console_scripts': [
                'nvidia_pyindex=nvidia_pyindex.cmdline:main',
            ],
        },
        platforms=["Linux"],
        include_package_data=True,
        packages=find_packages(),
    )
